/**
 * Copyright (C) 2003-2008 eXo Platform SAS.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see<http://www.gnu.org/licenses/>.
 */

package org.exoplatform.services.rest.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.ws.rs.core.MultivaluedMap;

/**
 * @see MultivaluedMap
 * @author <a href="mailto:andrew00x@gmail.com">Andrey Parfonov</a>
 * @version $Id: $
 */
public class MultivaluedMapImpl extends HashMap<String, List<String>> implements
    MultivaluedMap<String, String> {

  /**
   * Generated by Eclipse.
   */
  private static final long serialVersionUID = -6066678602537059655L;

  /**
   * {@inheritDoc}
   */
  public void add(String key, String value) {
    if (value == null)
      return;
    
    List<String> list = getList(key);
    list.add(value);
  }

  /**
   * {@inheritDoc}
   */
  public String getFirst(String key) {
    List<String> list = get(key);
    return list != null && list.size() > 0 ? list.get(0) : null;
  }

  /**
   * {@inheritDoc}
   */
  public void putSingle(String key, String value) {
    if (value == null)
      return;

    List<String> list = getList(key);
    list.clear();
    list.add(value);
  }

  /**
   * Get List with specified key. If List does not exist new one be created. 
   * @param key a key.
   * @return List&lt;String&gt;
   */
  private List<String> getList(String key) {
    List<String> list = get(key);
    if (list == null) {
      list = new ArrayList<String>();
      put(key, list);
    }
    return list;
  }

}
