/**
 * Copyright (C) 2003-2008 eXo Platform SAS.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see<http://www.gnu.org/licenses/>.
 */

package org.exoplatform.services.rest.impl.resource;

import java.util.TreeMap;

import org.exoplatform.services.rest.impl.uri.UriPattern;

/**
 * Keeps collections of {@link ResourceMethodMap} for each {@link UriPattern}.
 * Map is sorted by rule: UriPattern with more characters is first, See
 * {@link UriPattern#URIPATTERN_COMPARATOR}.
 * 
 * @author <a href="mailto:andrew00x@gmail.com">Andrey Parfonov</a>
 * @version $Id: $
 */
public class SubResourceMethodMap extends TreeMap<UriPattern, ResourceMethodMap> {

  /**
   * Generated by Eclipse.
   */
  private static final long serialVersionUID = -3743127892583440219L;

  /**
   * Constructs new instance of SubResourceMethodMap with
   * comparator, see{@link UriPattern#URIPATTERN_COMPARATOR}.
   */
  public SubResourceMethodMap() {
    super(UriPattern.URIPATTERN_COMPARATOR);
  }

  /**
   * Find {@link ResourceMethodMap} by key {@link UriPattern}, if no one found
   * then new {@link ResourceMethodMap} is created.
   * 
   * @param uriPattern See {@link UriPattern}
   * @return ResourceMethodMap which was found by key or newly created
   */
  public ResourceMethodMap getMethodMap(UriPattern uriPattern) {
    ResourceMethodMap m = get(uriPattern);
    if (m == null) {
      m = new ResourceMethodMap();
      put(uriPattern, m);
    }
    return m;
  }

}
