/**
 * Copyright (C) 2003-2007 eXo Platform SAS.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see<http://www.gnu.org/licenses/>.
 */

package org.exoplatform.services.rest.servlet;

import java.io.IOException;
import java.io.OutputStream;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.ext.MessageBodyWriter;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.ExoContainerContext;
import org.exoplatform.services.rest.Connector;
import org.exoplatform.services.rest.ContainerResponseWriter;
import org.exoplatform.services.rest.GenericContainerResponse;
import org.exoplatform.services.rest.RequestHandler;
import org.exoplatform.services.rest.impl.ContainerResponse;
import org.exoplatform.services.rest.impl.EnvironmentContext;
import org.exoplatform.services.rest.impl.header.HeaderHelper;

/**
 * This servlet is front-end for the REST engine.
 * 
 * @author <a href="mailto:andrew00x@gmail.com">Andrey Parfonov</a>
 * @version $Id: $
 */
public class RestServlet extends HttpServlet implements Connector {

  /**
   * Generated by Eclipse.
   */
  private static final long serialVersionUID = 2152962763071591181L;

  /**
   * See {@link ServletConfig}.
   */
  private ServletConfig     config;

  /**
   * See {@link ServletContext}.
   */
  private ServletContext    context;

  /**
   * {@inheritDoc}
   */
  @Override
  public void init(ServletConfig config) {
    this.config = config;
    this.context = config.getServletContext();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public void service(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws IOException,
                                                                                       ServletException {
    // Current container must be set by filter.

    ExoContainer container = ExoContainerContext.getCurrentContainer();

    RequestHandler requestHandler = (RequestHandler) container.getComponentInstanceOfType(RequestHandler.class);

    EnvironmentContext env = new EnvironmentContext();
    env.put(HttpServletRequest.class, httpRequest);
    env.put(HttpServletResponse.class, httpResponse);
    env.put(ServletConfig.class, config);
    env.put(ServletContext.class, context);

    EnvironmentContext.setCurrent(env);

    ServletContainerRequest request = new ServletContainerRequest(httpRequest);
    ContainerResponse response = new ContainerResponse(new ServletContainerResponseWriter(httpResponse));
    requestHandler.handleRequest(request, response);

    EnvironmentContext.setCurrent(null);
  }

  /**
   * See {@link ContainerResponseWriter}.
   */
  class ServletContainerResponseWriter implements ContainerResponseWriter {

    /**
     * See {@link HttpServletResponse}.
     */
    private HttpServletResponse servletResponse;

    /**
     * @param response HttpServletResponse
     */
    ServletContainerResponseWriter(HttpServletResponse response) {
      this.servletResponse = response;
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public void writeBody(GenericContainerResponse response, MessageBodyWriter entityWriter) throws IOException {
      Object entity = response.getEntity();
      if (entity != null) {
        OutputStream out = servletResponse.getOutputStream();
        entityWriter.writeTo(entity,
                             entity.getClass(),
                             response.getEntityType(),
                             null,
                             response.getContentType(),
                             response.getHttpHeaders(),
                             out);
        out.flush();
      }
    }

    /**
     * {@inheritDoc}
     */
    public void writeHeaders(GenericContainerResponse response) throws IOException {
      if (servletResponse.isCommitted())
        return;

      servletResponse.setStatus(response.getStatus());

      if (response.getHttpHeaders() != null) {
        // content-type and content-length should be preset in headers
        for (Map.Entry<String, List<Object>> e : response.getHttpHeaders().entrySet()) {
          String name = e.getKey();
          for (Object o : e.getValue()) {
            String value = null;
            if (o != null && (value = HeaderHelper.getHeaderAsString(o)) != null)
              servletResponse.addHeader(name, value);
          }
        }
      }
    }
  }

}
